#
# browser.py
# $Id: browser.py,v 1.13 1996/10/15 21:36:15 kzeil Exp $
#
# Koert Zeilstra - Network Solutions
# 1996.09.04
#

import os
import string
import Tkinter

import rwhois
import canvtree
import recordinfo
import recordedit
import errordlg
import icondata

#
# RecordBrowserToplevel
#
#
class RecordBrowserToplevel(Tkinter.Toplevel):
  def __init__ (self, parent):
    Tkinter.Toplevel.__init__ (self, parent)
    self._close_command = None

    self._browser = RecordBrowser(self)
    self._browser.pack(side="top", fill="both", expand=1)

    self._info_entry = Tkinter.Entry(self, {"width": 40})
    self._info_entry.pack({"side": "top", "fill": "x"})

    buttons = Tkinter.Frame(self)
    self._close_button = Tkinter.Button(buttons, {"text": "Close",
		"width": 8, "command": self.close})
    self._close_button.pack ({"side": "right", "padx": 3, "pady": 2})
    buttons.pack ({"side": "top", "fill": "x", "pady": 1})

    self.protocol("WM_DELETE_WINDOW", self.close)

  def closeCommand (self, command):
    self._close_command = command

  def close (self):
    if self._close_command != None:
      self._close_command()

  def showInfo (self, message):
    self._info_entry.delete(0, "end")
    self._info_entry.insert(0, message)
    self.update_idletasks()

  def showRecords (self, records, schema_cache):
    self._browser.showRecords(records, schema_cache)

#
# RecordBrowser
#
#
class RecordBrowser(Tkinter.Frame):
  def __init__ (self, parent):
    Tkinter.Frame.__init__ (self, parent)
    self._query_command = None
    self._preferences = {}
    self._schema_cache = None
    self._record_info_dlg = None
    self._record_edit_dlg = None
    self._error_dlg = None
    self._colors = {"read": "yellow", "normal": "black",
		"select": "blue"}
    self._type_font = "-*-helvetica-bold-r-normal-*-14-*-*-*-*-*-*-*"
    self._author_font = "-*-helvetica-medium-r-normal-*-14-*-*-*-*-*-*-*"
    self._line_height = 12
    self._x_indent = 15
    self._x_margin = 30
    self._y_margin = 5
    self._info_icon = Tkinter.Image("bitmap", data=icondata.info_icon)
    self._edit_icon = Tkinter.Image("bitmap", data=icondata.edit_icon)
    self._folder_closed = Tkinter.Image("bitmap", data=icondata.arrow_right)
    self._folder_open = Tkinter.Image("bitmap", data=icondata.arrow_down)

    view = Tkinter.Frame(self)
    self._canvas = Tkinter.Canvas(view, {"width": "300",
		"height": "300",
		"scrollregion": "0 0 400 300",
		"relief": "sunken", "borderwidth": 2})
    xscroll = Tkinter.Scrollbar(view, {"orient": "horizontal",
		"command": self._canvas.xview})
    yscroll = Tkinter.Scrollbar(view, {"orient": "vertical",
		"command": self._canvas.yview})
    self._canvas["xscrollcommand"] = xscroll.set
    self._canvas["yscrollcommand"] = yscroll.set
    xscroll.pack({"side": "bottom", "fill": "x"})
    yscroll.pack({"side": "right", "fill": "y"})
    self._canvas.pack({"fill": "both", "expand": 1})
    view.pack({"fill": "both", "expand": 1})

    self._canvas.bind("<Any-ButtonPress-1>", self.button1Clicked)
    self._canvas.bind("<2>", self.canvasScan)
    self._canvas.bind("<B2-Motion>", self.canvasDragto)

  def canvasScan (self, event):
    self._canvas.scan_mark(event.x, event.y)

  def canvasDragto (self, event):
    self._canvas.scan_dragto(event.x, event.y)

  def queryCommand (self, command):
    self._query_command = command

  def printToFile (self, filename):
    self._canvas.postscript(file=filename)

  def clearAll (self):
    self._records = []
    self._tree = canvtree.Tree()
    self._tree_items = {}

    self.displayTree()

  def setPreferences(self, preferences):
    self._preferences = preferences

  def showRecords (self, records, schema_cache):
    self._schema_cache = schema_cache
    self._records = records
    self._tree = canvtree.Tree()
    self._tree_items = {}

    categories = self.getRecordCategories(self._records)
    types = categories.keys()

    for type in types:
      label_tags = "label type %s" % type
      images = [(self._folder_closed, "image type %s" % type)]
      item = canvtree.TreeItem(type, label_tags, images)
      self._tree.addItem(item)
      self._tree_items["type %s" % type] = item

    self.displayTree()

  def displayTree (self):
    canvas_height = self._tree.getHeight()
    self._canvas["scrollregion"] = "0 0 400 %d" % canvas_height
    self._canvas.delete("all")

    self._tree.display(self._canvas, self._x_margin, self._line_height)

  def button1Clicked (self, event):
    x = event.x
    y = event.y
    tags = self._canvas.gettags("current")
    #print "tags", tags
    if len(tags) > 0:
      tag_type = tags[0]
      if tag_type == "image":
        type = tags[1]
        if type == "type":
          self.clickedTypeTree(tags[2])
          self.displayTree()
        if type == "record":
          self.clickedRecordTree(tags[2])
          self.displayTree()
        if type == "field":
          self.clickedReferralTree(tags[2])
          self.displayTree()
      elif tag_type == "info":
        type = tags[1]
        if type == "type":
          pass
        if type == "record":
          self.showRecordInfo(string.atoi(tags[2]))
        if type == "field":
          pass
      elif tag_type == "edit":
        type = tags[1]
        if type == "type":
          pass
        if type == "record":
          self.editRecord(string.atoi(tags[2]))
        if type == "field":
          pass

  def clickedTypeTree(self, record_type):
    tree_item = self._tree_items["type %s" % record_type]
    if tree_item.getSubtree():
      images = [(self._folder_closed, "image type %s" % record_type)]
      tree_item.setImages(images)
      tree_item.setSubtree(None)
    else:
      record_list = self.filterRecordsType(record_type)
      tree = canvtree.Tree()
      self.fillRecordsTree(record_list, tree)
      images = [(self._folder_open, "image type %s" % record_type)]
      tree_item.setImages(images)
      tree_item.setSubtree(tree)

  def fillRecordsTree (self, records, tree):
      for record, record_no in records:
        handle = record.getId()
        label_tags = "label record %d" % record_no
        images = [(self._folder_closed, "image record %s" % record_no),
		(self._info_icon, "info record %s" % record_no)]
        item = canvtree.TreeItem(handle, label_tags, images)
        tree.addItem(item)
        self._tree_items["record %d" % record_no] = item

  def clickedRecordTree(self, record_no):
    record_no = string.atoi(record_no)
    tree_item = self._tree_items["record %d" % record_no]
    if tree_item.getSubtree():
      images = [(self._folder_closed, "image record %s" % record_no),
		(self._info_icon, "info record %s" % record_no)]
      tree_item.setImages(images)
      tree_item.setSubtree(None)
    else:
      tree = canvtree.Tree()
      record = self._records[record_no]
      self.fillRecordItemsTree(record, record_no, tree)
      images = [(self._folder_open, "image record %s" % record_no),
		(self._info_icon, "info record %s" % record_no),
		(self._edit_icon, "edit record %s" % record_no)]
      tree_item.setImages(images)
      tree_item.setSubtree(tree)

  def fillRecordItemsTree(self, record, record_no, tree):
    schema = self._schema_cache.getSchemaDictOfRecord(record)
    record_handle = record.getId()
    names = record.getFieldNames()
    field_no = 0
    for name in names:
      field_type = schema[name].getType()
      contents_list = record.getField(name)
      i = 0
      for contents in contents_list:
        label = "%s: %s" % (name, contents)
        key = "%d.%d.%d" % (record_no, field_no, i)
        label_tags = "label field %s" % key
        if field_type == "SEE-ALSO":
          images = [(self._folder_closed, "image field %s" % key)]
        else:
          images = []
        item = canvtree.TreeItem(label, label_tags, images)
        tree.addItem(item)
        self._tree_items["field %s" % key] = item
        i = i + 1
      field_no = field_no + 1

  def clickedReferralTree(self, tag):
    key = tag
    items = string.split(tag, ".")
    record_no = string.atoi(items[0])
    field_no = string.atoi(items[1])
    content_no = string.atoi(items[2])
    tree_item = self._tree_items["field %s" % key]
    if tree_item.getSubtree():
      images = [(self._folder_closed, "image field %s" % key)]
      tree_item.setImages(images)
      tree_item.setSubtree(None)
    else:
      record = self._records[record_no]
      names = record.getFieldNames()
      contents = record.getField(names[field_no])[content_no]
      item1, item2, item3 = rwhois.parseColonTuple(contents)
      if item1 == "external":
        if item2 == "url":
          os.system("netscape %s &" % item3)
      elif item1 == "internal":
        tree = canvtree.Tree()
        query = "%s %s" % (item2, item3)
        records = self.queryReferral(record.getHost(), record.getPort(),
			query)
        if len(records) > 0:
          record_no = len(self._records)
          record_list = []
          for record in records:
            self._records.append(record)
            record_list.append((record, record_no))
            record_no = record_no + 1
          self.fillRecordsTree(record_list, tree)
          images = [(self._folder_open, "image field %s" % key)]
          tree_item.setImages(images)
          tree_item.setSubtree(tree)
      else:
        if item3[0:6] == "rwhois":
          tree = canvtree.Tree()
          query = item3[7:]
          records = self.queryReferral(item1, string.atoi(item2), query)
          record_no = len(self._records)
          record_list = []
          for record in records:
            self._records.append(record)
            record_list.append((record, record_no))
            record_no = record_no + 1
          self.fillRecordsTree(record_list, tree)
          images = [(self._folder_open, "image field %s" % key)]
          tree_item.setImages(images)
          tree_item.setSubtree(tree)

  def showRecordInfo(self, record_no):
    record = self._records[record_no]
    if self._record_info_dlg == None:
      self._record_info_dlg = recordinfo.RecordInfoDialog(self)
      self._record_info_dlg.closeCommand(self._record_info_dlg.withdraw)
    else:
      self._record_info_dlg.deiconify()
    self._record_info_dlg.showRecord(record)

  def editRecord(self, record_no):
    record = self._records[record_no]
    if self._record_edit_dlg == None:
      self._record_edit_dlg = recordedit.RecordEditDialog(self,
		schemacache=self._schema_cache)
      self._record_edit_dlg.okCommand(self.okEditRecord)
      self._record_edit_dlg.closeCommand(self._record_edit_dlg.withdraw)
      self._record_edit_dlg.showRecord(record, record_no)
    else:
      self._record_edit_dlg.showRecord(record, record_no)
      self._record_edit_dlg.deiconify()

  def okEditRecord(self):
    old_record = self._record_edit_dlg.getOldRecord()
    new_record = self._record_edit_dlg.getNewRecord()
    record_no = self._record_edit_dlg.getRecordNumber()
    #old_record.printAll()
    #new_record.printAll()
    rw = rwhois.Rwhois(old_record.getHost(), old_record.getPort())
    rw.setEmailAddress("kzeil@internic.net")
    try:
      rw.registerModifyRecord(old_record, new_record)
      self._records[record_no] = new_record
      tree_item = self._tree_items["record %d" % record_no]
      tree = canvtree.Tree()
      self.fillRecordItemsTree(new_record, record_no, tree)
      images = [(self._folder_open, "image record %s" % record_no),
		(self._info_icon, "info record %s" % record_no),
		(self._edit_icon, "edit record %s" % record_no)]
      tree_item.setImages(images)
      tree_item.setSubtree(tree)
      self.displayTree()
    except rwhois.RegisterError, message:
      if self._error_dlg == None:
        self._error_dlg = errordlg.ErrorDialog(self)
      self._error_dlg.setMessage(message)
      self._error_dlg.deiconify()

  def filterRecordsType(self, type):
    record_list = []
    i = 0
    for record in self._records:
      if record.getSchemaName() == type:
        record_list.append((record, i))
      i = i + 1
    return record_list

  def filterRecordsHandle(self, handle):
    record_list = []
    i = 0
    for record in self._records:
      if record.getId() == handle:
        record_list.append((record, i))
    return record_list

  def getRecordCategories(self, records):
    categories = {}
    for record in records:
      type = record.getSchemaName()
      if not categories.has_key(type):
        categories[type] = []
      categories[type].append(record)
    return categories

  def queryReferral (self, host, port, query):
    if self._query_command:
      return self._query_command(host, port, query)
    else:
      return []
