#
# recordedit.py
# $Id: recordedit.py,v 1.4 1996/10/15 21:36:19 kzeil Exp $
#
# Koert Zeilstra - Network Solutions
# 1996.09.10
#

import string
import Tkinter

import labelentry

class RecordEditDialog(Tkinter.Toplevel):
  def __init__(self, parent, **args):
    Tkinter.Toplevel.__init__(self, parent)

    self.title('PyRwhois - Edit Record')
    self.iconname('PyRwhois')

    self._close_command = None
    self._ok_command = None
    self._fields = None
    self._text_fields = None
    self._record = None
    self._record_no = None
    self._schema_cache = None

    if args.has_key("schemacache"):
      self._schema_cache = args["schemacache"]

    self._type = labelentry.LabelEntry(self, "Record Type:", 10, 25)
    self._type.setEditable(0)
    self._type.pack(side="top", fill="x")
    self._handle = labelentry.LabelEntry(self, "ID:", 10, 25)
    self._handle.setEditable(0)
    self._handle.pack(side="top", fill="x")
    self._server_name = labelentry.LabelEntry(self, "Server:", 10, 35)
    self._server_name.setEditable(0)
    self._server_name.pack(side="top", fill="x")
    self._server_port = labelentry.LabelEntry(self, "Port:", 10, 6)
    self._server_port.setEditable(0)
    self._server_port.pack(side="top", fill="x")

    self._separator = Tkinter.Frame(self, height=2, bd=1, relief="sunken")
    self._separator.pack(side="top", fill="x")

    buttons = Tkinter.Frame(self)
    ok_btn = Tkinter.Button(buttons, text="OK", width=8,
				command=self.ok)
    ok_btn.pack(side="left", padx=5, pady=3)
    cancel_btn = Tkinter.Button(buttons, text="Cancel", width=8,
				command=self.close)
    cancel_btn.pack(side="left", padx=5, pady=3)
    buttons.pack(side="top")

    self.protocol("WM_DELETE_WINDOW", self.close)

  def close(self):
    if self._close_command:
      self._close_command()
    else:
      self.withdraw()

  def ok(self):
    if self._ok_command:
      self._ok_command()
    self.close()

  def closeCommand(self, command):
    self._close_command = command

  def okCommand(self, command):
    self._ok_command = command

  def showRecord(self, record, record_no):
    self._record = record
    self._record_no = record_no
    self._type.setEntry(record.getSchemaName())
    self._handle.setEntry(record.getId())
    self._server_name.setEntry(record.getHost())
    self._server_port.setEntry(record.getPort())

    if self._fields:
      self._fields.destroy()
    self._fields = Tkinter.Frame(self)
    self._text_fields = {}

    if self._schema_cache:
      schema = self._schema_cache.getSchemaDictOfRecord(record)
      names = schema.getFieldNames()
    else:
      names = record.getFieldNames()
    label_width = self.maxItemLength(names)
    for name in names:
      #field_type = schema[name].getType()
      if record.hasField(name):
        contents_list = record.getField(name)
        content_width = self.maxItemLength(contents_list)
        content_height = len(contents_list)
      else:
        content_height = 1
      frame = Tkinter.Frame(self._fields)
      label = Tkinter.Label(frame, text=name, width=label_width,
			anchor="ne")
      text = Tkinter.Text(frame, width=40, height=content_height)
      if record.hasField(name):
        text.insert("1.0", string.join(contents_list, "\n"))
      label.pack(side="left", fill="y")
      text.pack(side="left")
      frame.pack(side="top", fill="x")
      self._text_fields[name] = text

    self._fields.pack(after=self._separator, side="top", fill="x")

  def maxItemLength(self, list):
    max = 0
    for item in list:
      if len(item) > max:
        max = len(item)
    return max

  def getOldRecord(self):
    return self._record

  def getNewRecord(self):
    record = self._record.getCopy()
    field_names = self._record.getFieldNames()
    for field_name in field_names:
      contents = self._text_fields[field_name].get("1.0", "end")
      if contents[-1] == "\n":
        contents = contents[:-1]
      record.setField(field_name, string.split(contents, "\n"))
    return record

  def getRecordNumber(self):
    return self._record_no
