#
# recordnew.py
# $Id: recordnew.py,v 1.1 1996/10/15 21:36:20 kzeil Exp $
#
# Koert Zeilstra - Network Solutions
# 1996.10.09
#

import string
import Tkinter

import rwhois
import dialog
import labelentry

class RecordFrame(Tkinter.Frame):
  def __init__(self, parent):
    Tkinter.Frame.__init__(self, parent)
    self._close_command = None
    self._ok_command = None

  def close(self):
    if self._close_command:
      self._close_command()
    else:
      self.withdraw()

  def ok(self):
    if self._ok_command:
      self._ok_command()

  def closeCommand(self, command):
    self._close_command = command

  def okCommand(self, command):
    self._ok_command = command

  def setCommands(self, ok_cmd, close_cmd):
    self._ok_command = ok_cmd
    self._close_command = close_cmd

class RecordServerFrame(RecordFrame):
  def __init__(self, parent):
    Tkinter.Frame.__init__(self, parent)

    self._server_name = labelentry.LabelEntry(self, "Server:", 10, 35)
    self._server_name.pack(side="top", fill="x", pady=20)
    self._server_port = labelentry.LabelEntry(self, "Port:", 10, 6)
    self._server_port.pack(side="top", fill="x", pady=20)

    self._separator = Tkinter.Frame(self, height=2, bd=1, relief="sunken")
    self._separator.pack(side="top", fill="x")

    buttons = Tkinter.Frame(self)
    next_btn = Tkinter.Button(buttons, text="Next", width=8,
				command=self.ok)
    next_btn.pack(side="left", padx=5, pady=3)
    cancel_btn = Tkinter.Button(buttons, text="Cancel", width=8,
				command=self.close)
    cancel_btn.pack(side="left", padx=5, pady=3)
    buttons.pack(side="top")

  def getServerPort(self):
    port = string.atoi(self._server_port.getEntry())
    return self._server_name.getEntry(), port

class RecordSchemaFrame(RecordFrame):
  def __init__(self, parent):
    Tkinter.Frame.__init__(self, parent)

    self._server_name = labelentry.LabelEntry(self, "Server:", 10, 35)
    self._server_name.setEditable(0)
    self._server_name.pack(side="top", fill="x")
    self._server_port = labelentry.LabelEntry(self, "Port:", 10, 6)
    self._server_port.setEditable(0)
    self._server_port.pack(side="top", fill="x")

    frame = Tkinter.Frame(self)
    label = Tkinter.Label(frame, text="Schema Name", anchor="nw")
    label.pack(side="left", fill="y")
    list_frame = Tkinter.Frame(frame)
    self._schema_list = Tkinter.Listbox(list_frame, relief= "sunken",
		bd= 2, width=25, height=10)
    self._schema_list.bind("<ButtonRelease-1>", self._clickList)
    scrollbar = Tkinter.Scrollbar(list_frame, relief= "sunken", bd= 2,
		command=self._schema_list.yview)
    scrollbar.pack(side="right", fill="y")
    self._schema_list.pack(side="left", expand=1, fill="both")
    self._schema_list["yscrollcommand"] = scrollbar.set
    list_frame.pack(side="left", fill="both", expand=1)
    frame.pack(side="top")

    self._separator = Tkinter.Frame(self, height=2, bd=1, relief="sunken")
    self._separator.pack(side="top", fill="x")

    buttons = Tkinter.Frame(self)
    next_btn = Tkinter.Button(buttons, text="Next", width=8,
				command=self.ok)
    next_btn.pack(side="left", padx=5, pady=3)
    cancel_btn = Tkinter.Button(buttons, text="Cancel", width=8,
				command=self.close)
    cancel_btn.pack(side="left", padx=5, pady=3)
    buttons.pack(side="top")

  def _clickList(self, event):
    index = self._schema_list.nearest(event.y)
    self._selected_schema = self._schema_list.get(index)

  def setServerPort(self, host, port):
    self._server_name.setEntry(host)
    self._server_port.setEntry(port)

    rw = rwhois.Rwhois(host, port)
    names = rw.getSchemaNames()
    self._schema_list.delete(0, "end")
    for name in names:
      self._schema_list.insert("end", name)

  def getSchemaName(self):
    return self._selected_schema

class RecordDataFrame(RecordFrame):
  def __init__(self, parent):
    Tkinter.Frame.__init__(self, parent)

    self._field_frames = []
    self._field_texts = {}
    self._field_names = []

    self._server_name = labelentry.LabelEntry(self, "Server:", 10, 35)
    self._server_name.setEditable(0)
    self._server_name.pack(side="top", fill="x")
    self._server_port = labelentry.LabelEntry(self, "Port:", 10, 6)
    self._server_port.setEditable(0)
    self._server_port.pack(side="top", fill="x")
    self._schema = labelentry.LabelEntry(self, "Schema:", 10, 35)
    self._schema.setEditable(0)
    self._schema.pack(side="top", fill="x")

    self._record_frame = Tkinter.Frame(self)
    self._record_frame.pack(side="top")

    self._separator = Tkinter.Frame(self, height=2, bd=1, relief="sunken")
    self._separator.pack(side="top", fill="x")

    buttons = Tkinter.Frame(self)
    next_btn = Tkinter.Button(buttons, text="OK", width=8,
				command=self.ok)
    next_btn.pack(side="left", padx=5, pady=3)
    cancel_btn = Tkinter.Button(buttons, text="Cancel", width=8,
				command=self.close)
    cancel_btn.pack(side="left", padx=5, pady=3)
    buttons.pack(side="top")

  def setServerPortSchema(self, host, port, schema):
    self._server_name.setEntry(host)
    self._server_port.setEntry(port)
    self._schema.setEntry(schema)

    if self._field_frames:
      for frame in self._field_frames:
        frame.destroy()
    self._field_texts = {}
    self._field_frames = []

    rw = rwhois.Rwhois(host, port)
    self._schema_dict = rw.getSchemaDict(schema)
    self._field_names = self._schema_dict.getFieldNames()
    max_length = 0
    for name in self._field_names:
      if len(name) > max_length:
        max_length = len(name)

    for name in self._field_names:
      frame = Tkinter.Frame(self._record_frame)
      if self._schema_dict[name].getRequired() == "ON":
        color = "red"
      else:
        color = "black"
      label = Tkinter.Label(frame, text=name, width=max_length,
			anchor="ne", foreground=color)
      if self._schema_dict[name].getMultiple() == "OFF":
        text = Tkinter.Entry(frame, width=40)
      else:
        text = Tkinter.Text(frame, width=40, height=1)
        #text.bind("<Delete>", self.updateTextHeight)
        #text.bind("<BackSpace>", self.updateTextHeight)
        text.bind("<Return>", self.updateTextHeight)
      label.pack(side="left", fill="y")
      text.pack(side="left")
      frame.pack(side="top")
      self._field_frames.append(frame)
      self._field_texts[name] = text

  def getRecord(self):
    host = self._server_name.getEntry()
    port = string.atoi(self._server_port.getEntry())
    schema = self._schema.getEntry()
    record = rwhois.Record(host, port, schema)
    for name in self._field_names:
      if self._schema_dict[name].getMultiple() == "OFF":
        contents = self._field_texts[name].get()
      else:
        contents = self._field_texts[name].get("1.0", "end")
      lines = string.split(contents, "\n")
      for line in lines:
        if line != "":
          record.addField(name, string.strip(line))
    return record

  def updateTextHeight(self, event):
    contents = event.widget.get("1.0", "end")
    height = string.atoi(event.widget["height"])
    lines = string.count(contents, "\n") + 1
    if (height < lines) or (height > lines+1):
      event.widget["height"] = lines

class RecordNewDialog(Tkinter.Toplevel):
  def __init__(self, parent):
    Tkinter.Toplevel.__init__(self, parent)
    self._preferences = {}
    self._error_dlg = None
    self._message_dlg = None
    self._close_command = None
    self._ok_command = None

    self.title('PyRwhois - New Record')
    self.iconname('PyRwhois')

    self._server_frame = RecordServerFrame(self)
    self._server_frame.setCommands(self.next, self.close)
    self._server_frame.pack(side="top", fill="both", expand=1)

    self._schema_frame = RecordSchemaFrame(self)
    self._schema_frame.setCommands(self.next, self.close)
    self._record_frame = RecordDataFrame(self)
    self._record_frame.setCommands(self.ok, self.close)

    self._current_frame = self._server_frame

    self.protocol("WM_DELETE_WINDOW", self.close)

  def next(self):
    if self._current_frame == self._server_frame:
      self._host, self._port = self._server_frame.getServerPort()
      self._schema_frame.setServerPort(self._host, self._port)
      self._server_frame.forget()
      self._schema_frame.pack(side="top")
      self._current_frame = self._schema_frame
    elif self._current_frame == self._schema_frame:
      self._schema = self._schema_frame.getSchemaName()
      self._record_frame.setServerPortSchema(self._host, self._port,
			self._schema)
      self._schema_frame.forget()
      self._record_frame.pack(side="top")
      self._current_frame = self._record_frame

  def ok(self):
    rw = rwhois.Rwhois(self._host, self._port)
    address = self._preferences["email"]
    rw.setEmailAddress(address)
    record = self._record_frame.getRecord()
    #record.printAll()
    try:
      id = rw.registerAddRecord(record)
      if self._message_dlg == None:
        self._message_dlg = dialog.Dialog(self)
      self._message_dlg.setMessage("Registration successful.\n\nNew record ID: %s" % id)
      self._message_dlg.deiconify()
    except rwhois.RegisterError, message:
      if self._error_dlg == None:
        self._error_dlg = dialog.ErrorDialog(self)
      self._error_dlg.setMessage(message)
      self._error_dlg.deiconify()
    else:
      
      if self._ok_command:
        self._ok_command()
      self.close()

  def close(self):
    if self._close_command:
      self._close_command()
    else:
      self.withdraw()

  def closeCommand(self, command):
    self._close_command = command

  def okCommand(self, command):
    self._ok_command = command

  def reset(self):
    if self._current_frame != self._server_frame:
      self._current_frame.forget()
      self._server_frame.pack(side="top")
      self._current_frame = self._server_frame

  def setPreferences(self, preferences):
    self._preferences = preferences
