#
# rwhois.py
#
# $Id: rwhois.py,v 1.15 1996/10/15 21:36:22 kzeil Exp $
#
# Rwhois interface
# Koert Zeilstra - Network Solutions
# 1996.08.22
#

import socket
import os
import string
import regex
import sys
import struct
import select
import crc

buffer_size = 1024
UDP_MAX_SIZE = 1024
UDP_TIMEOUT = 10 # seconds
NORMAL = 0
INFO_ON = 1

UdpTimeout = "UdpTimeout"
RegisterError = "RegisterError"

class Rwhois:
  def __init__(self, host, port):
    self._host = host
    self._port = port
    self._display = "dump"
    self._limit = 20
    self._cache = "off"
    self._forward = "off"
    self._language = "english"
    self._feedback_command = None
    self._parse = 1
    self._use_udp = 1
    self._udp_sequence = 0
    self._email_address = None

  def __del__(self):
    pass

  def getHost(self):
    return self._host
  def getPort(self):
    return self._port

  def setDisplay(self, display):
    self._display = display
  def setLimit(self, limit):
    self._limit = limit
  def setCache(self, cache):
    self._cache = cache
  def setForward(self, forward):
    self._forward = forward
  def setLanguage(self, language):
    self._language = language
  def setFeedbackCommand(self, command):
    self._feedback_command = command
  def setParse(self, parse):
    self._parse = parse
  def useUdp(self, flag):
    self._use_udp = flag
  def setEmailAddress(self, address):
    self._email_address = address

  def getSchemaNames(self):
    """ Gets all the schema names from the server, returns list of names
    """
    s = self.openTcpSocket(self._host, self._port)
    s.send("-display dump\n")
    s.send("-object\n")
    s.send("-quit\n")
    lines = self.collectLines(s)
    s.close()
    names = []
    for line in lines:
      if string.find(line, "%object ") == 0:
        sub = string.strip(line[8:])
        if sub != "":
          schema, field, value = self.parseSchemaTuple(line[8:])
          if not schema in names:
            names.append(schema)
    return names

  def getSchema(self, schema_name):
    s = self.openTcpSocket(self._host, self._port)
    s.send("-display dump\n")
    s.send("-schema %s\n" % schema_name)
    s.send("-quit\n")
    lines = self.collectLines(s)
    s.close()
    fields = []
    for line in lines:
      if string.find(line, "%schema ") == 0:
        sub = string.strip(line[8:])
        if sub != "":
          schema, field, value = self.parseSchemaTuple(line[8:])
          #print "--schema", schema, field, value
          if field == "attribute":
            fields.append(value)
  
    return fields

  def getSchemaDict(self, schema_name):
    s = self.openTcpSocket(self._host, self._port)
    s.send("-display dump\n")
    s.send("-schema %s\n" % schema_name)
    s.send("-quit\n")
    lines = self.collectLines(s)
    s.close()
    fields = {}
    schema_def = {}
    new_schema = Schema()
    for line in lines:
      #print "line", line
      if string.find(line, "%schema") == 0:
        sub = string.strip(line[8:])
        if sub == "":
          if fields.has_key("attribute"):
            new_field = SchemaField()
	    new_field.extractFields(fields)
            #new_field.printAll()
            schema_def[new_field.getName()] = new_field
            new_schema[new_field.getName()] = new_field
          fields = {}
        else:
          schema_name, field, value = self.parseSchemaTuple(line[8:])
          #print "--schema", schema, field, value
          fields[string.lower(field)] = value
    if fields.has_key("attribute"):
      new_field = SchemaField()
      new_field.extractFields(fields)
      schema_def[new_field.getName()] = new_field
      new_schema[new_field.getName()] = new_field
    #return schema_def
    return new_schema

  def query(self, query_string, host=None, port=None):
    if host == None:
      host = self._host
    if port == None:
      port = self._port
    do_tcp = 0
    if self._use_udp:
      try:
        records, errors = self.doUdpQuery(query_string, host, port)
        if errors:
          if string.find(errors[0], "600") < 5:
            do_tcp = 1
      except UdpTimeout:
        do_tcp = 1
    else:
      do_tcp = 1

    if do_tcp:
      return self.doTcpQuery(query_string, self._host, self._port)
    else:
      return records, errors

  def doQuery(self, query_string, host, port):
    do_tcp = 0
    if self._use_udp:
      try:
        records, errors = self.doUdpQuery(query_string, host, port)
        if errors:
          if string.find(errors[0], "600") < 5:
            do_tcp = 1
      except UdpTimeout:
        do_tcp = 1
    else:
      do_tcp = 1

    if do_tcp:
      return self.doTcpQuery(query_string, host, port)
    else:
      return records, errors

  def doUdpQuery(self, query_string, host, port):
    if self._parse:
      display = "dump"
    else:
      display = self._display
    if self._feedback_command:
      self._feedback_command("Sending UDP packet to %s %d" % (host, port))
    s = self.openUdpSocket(host, port)
    self._udp_sequence = self._udp_sequence + 1
    buffer1 = "-RWhois" + chr(0) + struct.pack("cchh", chr(1), chr(0),
		self._udp_sequence, UDP_MAX_SIZE)
    buffer2 = "-display %s\n-limit %d\n%s\n" % (display, self._limit,
		query_string)
    buffer = buffer1 + struct.pack("h", 0) + buffer2
    checksum = crc.crc_16(buffer, 0)
    print "checksum", checksum
    buffer = buffer1 + struct.pack("h", checksum) + buffer2
    for c in buffer:
      print "%d - %s" % (ord(c), c)
    print "buffer -%s-" % buffer
    bytes = s.sendto(buffer, (host, port))
    retry_count = 0
    done = 0
    while not done and retry_count < 5:
      read_list, write_list, cond_list = select.select([s], [], [], UDP_TIMEOUT)
      if read_list:
        data, address = s.recvfrom(UDP_MAX_SIZE+64)
        cookie = data[0:8]
        print "packet", data
        major, minor, recv_sequence, checksum = struct.unpack("cchh", data[8:14])
        payload = data[12:]
        if string.find(cookie, "%RWhois") != 0:
          print "cookie", cookie
          if self._feedback_command:
            self._feedback_command("Received bad cookie")
        elif self._udp_sequence != recv_sequence:
          if self._feedback_command:
            self._feedback_command("Received wrong sequence number")
        else:
          done = 1
      else:
        retry_count = retry_count + 1
        if self._feedback_command:
          self._feedback_command("UDP timeout %d: %s %d" % (retry_count,
			host, port))
    s.close()
    if not done:
      raise UdpTimeout

    lines = string.split(payload, "\n")
    if self._parse:
      return self.parseLines(lines, query_string, host, port)
    else:
      return lines

  def doTcpQuery(self, query_string, host, port):
    if self._parse:
      display = "dump"
    else:
      display = self._display

    s = self.openTcpSocket(host, port)
    s.send("-display %s\n" % display)
    s.send("-limit %d\n" % self._limit)
    #s.send("-cache %s\n" % self._cache)
    #s.send("-forward %s\n" % self._forward)
    #s.send("-language %s\n" % self._language)
    s.send("%s\n" % query_string)
    lines = self.collectLines(s)
    s.close()

    if self._parse:
      return self.parseLines(lines, query_string, host, port)
    else:
      return lines

  def parseLines(self, lines, query_string, host, port):
    error_list = []
    record = []
    record_list = []
    line_mode = NORMAL
    for line in lines:
      if line_mode == NORMAL:
        if len(line) < 1:
          # Empty line, indicates end of record
          if len(record) > 0:
            new_record = self.parseRecordLines(record, host, port)
            if new_record.getSchemaName() == "referral":
              nrecords, nerrors = self.retrieveReferralRecord(new_record,
			query_string)
              record_list = record_list + nrecords
              error_list = error_list + nerrors
            else:
              record_list.append(new_record)
          record = []
        else:
          if string.find(line, "%") == 0:
            # line is special message
            if string.find(line, "%error") == 0:
              error_list.append(line[7:])
            if string.find(line, "%info on") == 0:
              line_mode = INFO_ON
          else:
            # line is record line
            record.append(line)
      elif line_mode == INFO_ON:
        if string.find(line, "%info off") == 0:
          line_mode = NORMAL
    if len(record) > 0:
      new_record = self.parseRecordLines(record, host, port)
      if new_record.getSchemaName() == "referral":
        nrecords, nerrors = self.retrieveReferralRecord(new_record,
		query_string)
        record_list = record_list + nrecords
        error_list = error_list + nerrors
      else:
        record_list.append(new_record)
    return record_list, error_list

  def retrieveReferralRecord(self, record, query_string):
    referral = record.getField("Referral")
    host, port, protocol = self.parseSchemaTuple(referral[0])
    port = string.atoi(port)
    #print "referral", host, port, protocol
    #rw = Rwhois(host, port)
    #return rw.query(query_string)
    return self.doQuery(query_string, host, port)

  def registerAddRecord(self, new_record):
    host = new_record.getHost()
    port = new_record.getPort()
    s = self.openTcpSocket(host, port)
    line = fetchLineFromSocket(s)
    s.send("-register on add %s\n" % self._email_address)
    #print "-register on add %s\n" % self._email_address
    line = fetchLineFromSocket(s)
    if string.find(line, "%ok") != 0:
      raise RegisterError, line
    s.send("Object-Type: %s\n" % new_record.getSchemaName())
    field_names = new_record.getFieldNames()
    for field_name in field_names:
      content_list = new_record.getField(field_name)
      for content in content_list:
        s.send("%s: %s\n" % (field_name, content))
    s.send("-register off\n")
    line = fetchLineFromSocket(s, 2)
    lines = string.split(line, "\n")
    id = None
    if len(lines) > 1:
      i = string.find(lines[1], ":")
      if i != -1:
        id = string.strip(lines[1][i+1:])
    #print lines
    s.close()
    if string.find(line, "%ok") != 0:
      raise RegisterError, line
    return id
    
  def registerModifyRecord(self, old_record, new_record):
    host = old_record.getHost()
    port = old_record.getPort()
    s = self.openTcpSocket(host, port)
    line = fetchLineFromSocket(s)
    s.send("-register on mod %s\n" % self._email_address)
    line = fetchLineFromSocket(s)
    if string.find(line, "%ok") != 0:
      raise RegisterError, line
    s.send("_OLD_\n")
    s.send("Object-Type: %s\n" % old_record.getSchemaName())
    field_names = old_record.getFieldNames()
    for field_name in field_names:
      content_list = old_record.getField(field_name)
      for content in content_list:
        s.send("%s: %s\n" % (field_name, content))
    s.send("_NEW_\n")
    s.send("Object-Type: %s\n" % new_record.getSchemaName())
    field_names = new_record.getFieldNames()
    for field_name in field_names:
      content_list = new_record.getField(field_name)
      for content in content_list:
        s.send("%s: %s\n" % (field_name, content))
    s.send("-register off\n")
    line = fetchLineFromSocket(s)
    s.close()
    if string.find(line, "%ok") != 0:
      raise RegisterError, line
    
  def openTcpSocket(self, host, port):
    if self._feedback_command:
      self._feedback_command("Connecting TCP to %s %d" % (host, port))
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect(host, port)
    s.send("-RWhois V-1.0:001fff:0f:0f [InterNIC B0.1]\n")
    return s

  def openUdpSocket(self, host, port):
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.bind("", 0)
    return s

  def collectLines(self, s):
    lines = []
    data = s.recv(buffer_size)
    while data:
      if data[0] == "\n":
        #print "--first", data
        data = data[1:]
      tmp_lines = string.split(data, "\n")
      lines = lines + tmp_lines[:-1]
      if data and data[-1] == "\n":
        #print "* -%s-" % data
        data = "\n" + s.recv(buffer_size)
      else:
        data = tmp_lines[-1] + s.recv(buffer_size)

    return lines

  def parseSchemaTuple(self, line):
    schema = ""
    field = ""
    value = ""
    #print "parseSchemaTuple", line
    i1 = string.find(line, ":", 0)
    if i1 != -1:
      schema = line[:i1]
      i2 = string.find(line, ":", i1+1)
      if i2 != -1:
        field = line[i1+1:i2]
        value = line[i2+1:]
    return schema, field, value

  def parseRecordLines(self, lines, host, port):
    schema, field, value = self.parseSchemaTuple(lines[0])
    new_record = Record(host, port, schema)
    for line in lines:
      schema, field, value = self.parseSchemaTuple(line)
      new_record.addField(field, value)
    return new_record

def fetchLineFromSocket(sock, no_of_lines=1):
  data = sock.recv(buffer_size)
  line = data
  #while data[-1] != "\n":
  while string.count(line, "\n") < no_of_lines:
    data = sock.recv(buffer_size)
    line = line + data
  return line

def parseColonTuple(line):
  schema = ""
  field = ""
  value = ""
  #print "parseSchemaTuple", line
  i1 = string.find(line, ":", 0)
  if i1 != -1:
    schema = line[:i1]
    i2 = string.find(line, ":", i1+1)
    if i2 != -1:
      field = line[i1+1:i2]
      value = line[i2+1:]
  return schema, field, value

class SchemaField:
  def __init__(self):
    self._name = None
    self._description = None
    self._indexed = None
    self._required = None
    self._multiple = None
    self._type = None
    self._format = None

  def set(self, name, description, indexed, required, multiple, type, format):
    self._name = name
    self._description = description
    self._indexed = indexed
    self._required = required
    self._multiple = multiple
    self._type = type
    self._format = format

  def extractFields(self, fields):
    self._name = fields["attribute"]
    self._description = fields["description"]
    if fields.has_key("indexed"):
      self._indexed = fields["indexed"]
    self._required = fields["required"]
    if fields.has_key("mult-line"):
      self._multiple = fields["mult-line"]
    self._type = fields["type"]
    if fields.has_key("format"):
      self._format = fields["format"]
    else:
      self._format = None

  def printAll(self):
    print "SchemaField", self._name, self._description, self._indexed, \
           self._required, self._multiple, self._type, self._format

  def getName(self):
    return self._name
  def getDescription(self):
    return self._description
  def getIndexed(self):
    return self._indexed
  def getRequired(self):
    return self._required
  def getMultiple(self):
    return self._multiple
  def getType(self):
    return self._type

class Schema:
  _field_names = []
  _field_dict = {}

  def __init__(self):
    pass

  def getFieldNames(self):
    return self._field_names

  def __setitem__(self, name, value):
    if not self._field_dict.has_key(name):
      self._field_names.append(name)
    self._field_dict[name] = value

  def __getitem__(self, name):
    return self._field_dict[name]

class Record:
  def __init__(self, host, port, schema_name):
    self._host = host
    self._port = port
    self._schema_name = schema_name
    self._fields = {} # dictionary <name>: <contents>
    self._field_names = []

  def getHost(self):
    return self._host
  def getPort(self):
    return self._port
  def getSchemaName(self):
    return self._schema_name

  def getId(self):
    if self._fields.has_key("ID"):
      return self._fields["ID"][0]
    elif self._fields.has_key("Handle"):
      return self._fields["Handle"][0]
    else:
      return ""

  def addField(self, name, contents):
    if self._fields.has_key(name):
      self._fields[name].append(contents)
    else:
      self._field_names.append(name)
      self._fields[name] = [contents]
  def getFieldNames(self):
    return self._field_names
  def getFieldValues(self):
    return self._fields.values()
  def hasField(self, name):
    return self._fields.has_key(name)
  def getField(self, name):
    return self._fields[name]
  def setField(self, name, contents):
    if not name in self._field_names:
      self._field_names.append(name)
    self._fields[name] = contents

  def getCopy(self):
    record = Record(self._host, self._port, self._schema_name)
    record._fields = {}
    for key in self._fields.keys():
      record._fields[key] = self._fields[key]
    record._field_names = map(None, record._field_names)
    return record

  def printAll(self, prefix=""):
    print "%sSchema name: %s" % (prefix, self._schema_name)
    print "%sHost: %s" % (prefix, self._host)
    print "%sPort: %s" % (prefix, self._port)
    for field in self._fields.keys():
      print "%s  %s: %s" % (prefix, field, self._fields[field])

class SchemaCache:
  def __init__(self):
    self._schema_dict = {}

  def getSchemaDict(self, host, port, schema_name):
    if self._schema_dict.has_key((host, port, schema_name)):
      schema = self._schema_dict[(host, port, schema_name)]
    else:
      rw = Rwhois(host, port)
      schema = rw.getSchemaDict(schema_name)
      self._schema_dict[(host, port, schema_name)] = schema
    return schema

  def getSchemaDictOfRecord(self, record):
    return self.getSchemaDict(record.getHost(), record.getPort(),
		record.getSchemaName())


def print_help():
  print "Python rwhois client - Version 0.2 (Beta)"
  print "usage: %s [options]" % sys.argv[0]
  print "options:"
  print " -host <hostname>     use <hostname> as rwhois server"
  print " -port <portnumber>   use port <portnumber>"
  print " -schemas             displays available schema names"
  print " -schema <name>       displays schema definition for <name>"
  print " -query <query>       executes <query> and displays results"

if __name__ == "__main__":
  host = "rwhois.internic.net"
  port = 4321
  query = None
  schemas = 0
  schema_names = []
  i = 1
  l = len(sys.argv)
  while i < l:
    arg = sys.argv[i]
    if arg == "-host":
      i = i+1
      host = sys.argv[i]
    elif arg == "-port":
      i = i+1
      port = string.atoi(sys.argv[i])
    elif arg == "-query":
      i = i+1
      query = sys.argv[i]
    elif arg == "-schemas":
      schemas = 1
    elif arg == "-schema":
      i = i+1
      schema_names.append(sys.argv[i])
    elif arg == "-help":
      print_help()
    else:
      query = arg
    i = i+1

  rw = Rwhois(host, port)

  if schemas == 1 or len(schema_names) > 0:
    names = rw.getSchemaNames()

  if schemas == 1:
    print "Schema names:"
    for name in names:
      print " ", name
  if len(schema_names) > 0:
    for name in schema_names:
      print "Schema %s:" % name
      fields = rw.getSchema(name)
      for field in fields:
        print "  ", field

  if query:
    print "Query %s:" % query
    records, errors = rw.query(query)
    if len(errors) > 0:
      print " Errors:"
      for error in errors:
        print " ", error
    print " Records:"
    for record in records:
      print "  -----------"
      record.printAll("  ")

